@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -20,22 +20,21 @@ package org.apache.hadoop.hive.ql.exec;
 
 import java.io.Serializable;
 import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
 import java.util.TreeSet;
-import java.util.concurrent.Future;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.hive.ql.exec.tez.ReduceRecordSource;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hive.conf.HiveConf;
+import org.apache.hadoop.hive.ql.CompilationOpContext;
 import org.apache.hadoop.hive.ql.exec.persistence.RowContainer;
+import org.apache.hadoop.hive.ql.exec.tez.InterruptibleProcessing;
 import org.apache.hadoop.hive.ql.exec.tez.RecordSource;
-import org.apache.hadoop.hive.ql.exec.tez.ReduceRecordSource;
 import org.apache.hadoop.hive.ql.exec.tez.TezContext;
 import org.apache.hadoop.hive.ql.metadata.HiveException;
 import org.apache.hadoop.hive.ql.plan.CommonMergeJoinDesc;
@@ -69,7 +68,7 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
 
   private static final long serialVersionUID = 1L;
   private boolean isBigTableWork;
-  private static final Log LOG = LogFactory.getLog(CommonMergeJoinOperator.class.getName());
+  private static final Logger LOG = LoggerFactory.getLogger(CommonMergeJoinOperator.class.getName());
   transient List<Object>[] keyWritables;
   transient List<Object>[] nextKeyWritables;
   transient RowContainer<List<Object>>[] nextGroupStorage;
@@ -90,14 +89,22 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
       new ArrayList<Operator<? extends OperatorDesc>>();
   transient Set<Integer> fetchInputAtClose;
 
-  public CommonMergeJoinOperator() {
+  // A field because we cannot multi-inherit.
+  transient InterruptibleProcessing interruptChecker;
+
+  /** Kryo ctor. */
+  protected CommonMergeJoinOperator() {
     super();
   }
 
+  public CommonMergeJoinOperator(CompilationOpContext ctx) {
+    super(ctx);
+  }
+
   @SuppressWarnings("unchecked")
   @Override
-  public Collection<Future<?>> initializeOp(Configuration hconf) throws HiveException {
-    Collection<Future<?>> result = super.initializeOp(hconf);
+  public void initializeOp(Configuration hconf) throws HiveException {
+    super.initializeOp(hconf);
     firstFetchHappened = false;
     fetchInputAtClose = getFetchInputAtCloseList();
 
@@ -143,13 +150,28 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
 
     for (byte pos = 0; pos < order.length; pos++) {
       if (pos != posBigTable) {
-        fetchDone[pos] = false;
+        if ((parentOperators != null) && !parentOperators.isEmpty()
+            && (parentOperators.get(pos) instanceof TezDummyStoreOperator)) {
+          TezDummyStoreOperator dummyStoreOp = (TezDummyStoreOperator) parentOperators.get(pos);
+          fetchDone[pos] = dummyStoreOp.getFetchDone();
+        } else {
+          fetchDone[pos] = false;
+        }
       }
       foundNextKeyGroup[pos] = false;
     }
 
     sources = ((TezContext) MapredContext.get()).getRecordSources();
-    return result;
+    interruptChecker = new InterruptibleProcessing();
+
+    if (sources[0] instanceof ReduceRecordSource &&
+        parentOperators != null && !parentOperators.isEmpty()) {
+      // Tell ReduceRecordSource to flush last record as this is a reduce
+      // side SMB
+      for (RecordSource source : sources) {
+        ((ReduceRecordSource) source).setFlushLastRecord(true);
+      }
+    }
   }
 
   /*
@@ -219,7 +241,7 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
             continue;
           }
 
-          if (foundNextKeyGroup[i] == false) {
+          if (!foundNextKeyGroup[i]) {
             canEmit = false;
             break;
           }
@@ -247,13 +269,12 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
     // catch up with the big table.
     if (nextKeyGroup) {
       assert tag == posBigTable;
-      List<Byte> smallestPos = null;
+      List<Byte> listOfFetchNeeded = null;
       do {
-        smallestPos = joinOneGroup();
+        listOfFetchNeeded = joinOneGroup();
         //jump out the loop if we need input from the big table
-      } while (smallestPos != null && smallestPos.size() > 0
-          && !smallestPos.contains(this.posBigTable));
-
+      } while (listOfFetchNeeded != null && listOfFetchNeeded.size() > 0
+          && !listOfFetchNeeded.contains(this.posBigTable));
       return;
     }
 
@@ -349,6 +370,9 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
   }
 
   private void fetchNextGroup(Byte t) throws HiveException {
+    if (keyWritables[t] != null) {
+      return; // First process the current key.
+    }
     if (foundNextKeyGroup[t]) {
       // first promote the next group to be the current group if we reached a
       // new group in the previous fetch
@@ -368,21 +392,31 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
 
     // for tables other than the big table, we need to fetch more data until reach a new group or
     // done.
+    interruptChecker.startAbortChecks(); // Reset the time, we only want to count it in the loop.
     while (!foundNextKeyGroup[t]) {
       if (fetchDone[t]) {
         break;
       }
       fetchOneRow(t);
+      try {
+        interruptChecker.addRowAndMaybeCheckAbort();
+      } catch (InterruptedException e) {
+        throw new HiveException(e);
+      }
     }
     if (!foundNextKeyGroup[t] && fetchDone[t]) {
       this.nextKeyWritables[t] = null;
     }
   }
+  
+  @Override
+  public void close(boolean abort) throws HiveException {
+    joinFinalLeftData(); // Do this WITHOUT checking for parents
+    super.close(abort);
+  }
 
   @Override
   public void closeOp(boolean abort) throws HiveException {
-    joinFinalLeftData();
-
     super.closeOp(abort);
 
     // clean up
@@ -440,7 +474,6 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
       if (posBigTable != lastPos
           && (fetchInputAtClose.contains(lastPos)) && (fetchDone[lastPos] == false)) {
         // Do the join. It does fetching of next row groups itself.
-        LOG.debug("Calling joinOneGroup once again");
         ret = joinOneGroup();
       }
 
@@ -457,7 +490,7 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
     while (dataInCache) {
       for (byte pos = 0; pos < order.length; pos++) {
         if (this.foundNextKeyGroup[pos] && this.nextKeyWritables[pos] != null) {
-          promoteNextGroupToCandidate(pos);
+          fetchNextGroup(pos);
         }
       }
       joinOneGroup();
@@ -510,6 +543,10 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
     } else {
       int cmp = compareKeys(alias, key, keyWritable);
       if (cmp != 0) {
+        // Cant overwrite existing keys
+        if (nextKeyWritables[alias] != null) {
+          throw new HiveException("Attempting to overwrite nextKeyWritables[" + alias + "]");
+        }
         nextKeyWritables[alias] = key;
         return true;
       }
@@ -627,17 +664,19 @@ public class CommonMergeJoinOperator extends AbstractMapJoinOperator<CommonMerge
     if (parent == null) {
       throw new HiveException("No valid parents.");
     }
-    Map<Integer, DummyStoreOperator> dummyOps =
-        ((TezContext) (MapredContext.get())).getDummyOpsMap();
-    for (Entry<Integer, DummyStoreOperator> connectOp : dummyOps.entrySet()) {
-      if (connectOp.getValue().getChildOperators() == null
-          || connectOp.getValue().getChildOperators().isEmpty()) {
-        parentOperators.add(connectOp.getKey(), connectOp.getValue());
-        connectOp.getValue().getChildOperators().add(this);
+
+    if (parentOperators.size() == 1) {
+      Map<Integer, DummyStoreOperator> dummyOps =
+          ((TezContext) (MapredContext.get())).getDummyOpsMap();
+      for (Entry<Integer, DummyStoreOperator> connectOp : dummyOps.entrySet()) {
+        if (connectOp.getValue().getChildOperators() == null
+            || connectOp.getValue().getChildOperators().isEmpty()) {
+          parentOperators.add(connectOp.getKey(), connectOp.getValue());
+          connectOp.getValue().getChildOperators().add(this);
+        }
       }
     }
     super.initializeLocalWork(hconf);
-    return;
   }
 
   public boolean isBigTableWork() {
